@@ -2,9 +2,9 @@
 This module contains various useful functions.
 """
 
+import warnings
 from timeit import default_timer as timer
 from typing import Any, Callable, Dict, Iterator, List, Optional, Sequence, Union
-from warnings import warn
 
 import numpy as np
 import torch
@@ -20,16 +20,40 @@ class SlisemapWarning(Warning):
     pass
 
 
-def _assert(condition: bool, message: str):
+def _assert(condition: bool, message: str, method: Optional[Callable] = None):
     if not condition:
-        raise SlisemapException(message)
+        if method is None:
+            raise SlisemapException(f"AssertionError: {message}")
+        else:
+            raise SlisemapException(f"AssertionError, {method.__qualname__}: {message}")
+
+
+def _assert_no_trace(
+    condition: Callable[[], bool], message: str, method: Optional[Callable] = None
+):
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", category=torch.jit.TracerWarning)
+        _assert(condition(), message, method)
+
+
+def _deprecated(old: Callable, new: Optional[Callable] = None):
+    if new is None:
+        warnings.warn(
+            f"{old.__qualname__} is deprecated and may be removed in a future version",
+            DeprecationWarning,
+        )
+    else:
+        warnings.warn(
+            f"{old.__qualname__} is deprecated in favour of {new.__qualname__} and may be removed in a future version",
+            DeprecationWarning,
+        )
 
 
 def _warn(warning: str, method: Optional[Callable] = None):
     if method is None:
-        warn(warning, SlisemapWarning, 2)
+        warnings.warn(warning, SlisemapWarning, 2)
     else:
-        warn(f"{method.__qualname__}: {warning}", SlisemapWarning, 2)
+        warnings.warn(f"{method.__qualname__}: {warning}", SlisemapWarning, 2)
 
 
 def tonp(x: torch.Tensor) -> np.ndarray:
@@ -39,7 +63,7 @@ def tonp(x: torch.Tensor) -> np.ndarray:
         x (torch.Tensor): Input torch.Tensor.
 
     Returns:
-        np.ndarray: Output numpy.ndarray.
+        (np.ndarray): Output numpy.ndarray.
     """
     return x.cpu().detach().numpy()
 
@@ -53,7 +77,7 @@ class CheckConvergence:
 
     Args:
         patience (float, optional): How long should the optimisation continue without improvement. Defaults to 3.
-        max_iter (int, optional): The maximum number of iterations. Defaults to 2**30.
+        max_iter (int, optional): The maximum number of iterations. Defaults to `2**20`.
     """
 
     __slots__ = {
@@ -66,7 +90,7 @@ class CheckConvergence:
         "iter": "The current number of iterations.",
     }
 
-    def __init__(self, patience: float = 3, max_iter=1 << 30):
+    def __init__(self, patience: float = 3, max_iter=1 << 20):
         self.current = np.inf
         self.best = np.asarray(np.inf)
         self.counter = 0
@@ -82,15 +106,15 @@ class CheckConvergence:
     ) -> bool:
         """Check if the optimisation has converged.
 
-        If more than one loss value is provided, then only the first one is checked when storing the ``optimal_state``.
+        If more than one loss value is provided, then only the first one is checked when storing the `optimal_state`.
         The other losses are only used for checking convergence.
 
         Args:
             loss (Union[float, Sequence[float]]): The latest loss value(s).
-            store (Optional[Callable[[], Any]], optional): Function that returns the current state for storing in ``self.optimal_state``. Defaults to None.
+            store (Optional[Callable[[], Any]], optional): Function that returns the current state for storing in `self.optimal_state`. Defaults to None.
 
         Returns:
-            bool: True if the optimisation has converged.
+            (bool): True if the optimisation has converged.
         """
         self.iter += 1
         loss = np.asarray(loss)
@@ -130,7 +154,7 @@ def LBFGS(
         **kwargs (optional): Argumemts passed to `torch.optim.LBFGS`.
 
     Returns:
-        torch.optim.LBFGS: The LBFGS optimiser.
+        (torch.optim.LBFGS): The LBFGS optimiser.
     """
     optimiser = torch.optim.LBFGS(
         variables,
@@ -177,12 +201,12 @@ def PCA_rotation(
 
     Args:
         X (torch.Tensor): The original matrix.
-        components (int, optional): The maximum number of components in the embedding. Defaults to ``min(*X.shape)``.
+        components (int, optional): The maximum number of components in the embedding. Defaults to `min(*X.shape)`.
         full (bool, optional): Use a full SVD for the PCA (slower). Defaults to True.
         niter (int, optional): The number of iterations when a randomised approach is used. Defaults to 10.
 
     Returns:
-        torch.Tensor: Rotation matrix that turns the original matrix into the embedded space.
+        (torch.Tensor): Rotation matrix that turns the original matrix into the embedded space.
     """
     try:
         components = min(*X.shape, components) if components > 0 else min(*X.shape)
@@ -218,7 +242,7 @@ def global_model(
         ridge (float, optional): Ridge-regularisation coefficient for B ($\\lambda_{ridge} * ||B||_2$). Defaults to 0.0.
 
     Returns:
-        torch.Tensor: Global model coefficients.
+        (torch.Tensor): Global model coefficients.
     """
     shape = (1, X.shape[1] * Y.shape[1] if coefficients is None else coefficients)
     B = torch.zeros(shape, dtype=X.dtype, device=X.device).requires_grad_(True)
@@ -242,7 +266,7 @@ def dict_array(dict: Dict[str, Any]) -> Dict[str, np.ndarray]:
         dict (Dict[str, Any]): Dictionary.
 
     Returns:
-        Dict[str, np.ndarray]: The same dictionary where the values are numpy arrays with equal length.
+        (Dict[str, np.ndarray]): The same dictionary where the values are numpy arrays with equal length.
     """
     n = 1
     for k, v in dict.items():
@@ -258,14 +282,14 @@ def dict_array(dict: Dict[str, Any]) -> Dict[str, np.ndarray]:
 
 
 def dict_append(df: Dict[str, np.ndarray], d: Dict[str, Any]) -> Dict[str, np.ndarray]:
-    """Append a dictionary of values to a dictionary of numpy arrays (see ``dict_array``) inplace.
+    """Append a dictionary of values to a dictionary of numpy arrays (see `dict_array`) inplace.
 
     Args:
         df (Dict[str, np.ndarray]): Dictionary of numpy arrays.
         d (Dict[str, Any]): Dictionary to append.
 
     Returns:
-        Dict[str, np.ndarray]: The same dictionary as ``df`` with the values from ``d`` appended.
+        (Dict[str, np.ndarray]): The same dictionary as `df` with the values from `d` appended.
     """
     d = dict_array(d)
     for k in df.keys():
@@ -277,13 +301,13 @@ def dict_concat(
     dicts: Union[Sequence[Dict[str, Any]], Iterator[Dict[str, Any]]]
 ) -> Dict[str, np.ndarray]:
     """Combine multiple dictionaries into one by concatenating the values.
-    Calls ``dict_array`` to pre-process the dictionaries.
+    Calls `dict_array` to pre-process the dictionaries.
 
     Args:
         dicts (Union[Sequence[Dict[str, Any]], Iterator[Dict[str, Any]]]): Sequence or Generator with dictionaries (all must have the same keys).
 
     Returns:
-        Dict[str, np.ndarray]: Combined dictionary.
+        (Dict[str, np.ndarray]): Combined dictionary.
     """
     if isinstance(dicts, Sequence):
         dicts = (d for d in dicts)
@@ -291,3 +315,24 @@ def dict_concat(
     for d in dicts:
         dict_append(df, d)
     return df
+
+
+def _expand_variable_names(
+    variables: Sequence[str],
+    intercept: bool,
+    columns: int,
+    targets: Union[None, str, Sequence[str]],
+    coefficients: int,
+) -> List[str]:
+    if intercept and len(variables) == columns - 1:
+        variables = list(variables) + ["Intercept"]
+    if targets and not isinstance(targets, str) and len(targets) > 0:
+        if coefficients % len(variables) == 0 and coefficients % len(targets) == 0:
+            variables = [f"{t}: {v}" for t in targets for v in variables]
+            variables = variables[:coefficients]
+    _assert(
+        len(variables) == coefficients,
+        f"The number of variable names ({len(variables)}) must match the number of coefficients ({coefficients})",
+        _expand_variable_names,
+    )
+    return variables
